from enum import IntEnum


class NamedSymbol(IntEnum):
    none = 0
    A = 1
    B = 2
    C = 3
    D = 4
    E = 5
    F = 6
    G = 7
    H = 8
    I = 9
    J = 10
    K = 11
    L = 12
    M = 13
    N = 14
    O = 15
    P = 16
    Q = 17
    R = 18
    S = 19
    T = 20
    U = 21
    V = 22
    W = 23
    X = 24
    Y = 25
    Z = 26
    a = 27
    b = 28
    c = 29
    d = 30
    e = 31
    f = 32
    g = 33
    h = 34
    i = 35
    j = 36
    k = 37
    l = 38
    m = 39
    n = 40
    o = 41
    p = 42
    q = 43
    r = 44
    s = 45
    t = 46
    u = 47
    v = 48
    w = 49
    x = 50
    y = 51
    z = 52
    zero = 53
    one = 54
    two = 55
    three = 56
    four = 57
    five = 58
    six = 59
    seven = 60
    eight = 61
    nine = 62
    empty0 = 63
    circle = 64
    square = 65
    diamond = 66
    triangle_down = 67
    triangle_up = 68
    triangle_left = 69
    triangle_right = 70
    man = 71
    woman = 72
    cat = 73
    pawn = 74
    knight = 75
    bishop = 76
    rook = 77
    queen = 78
    king = 79
    arrow_right = 80
    arrow_left = 81
    arrow_up = 82
    arrow_down = 83
    star = 84
    droplet = 85
    heart = 86
    lightning = 87
    thoughtbubble = 88
    speechbubble = 89
    flower = 90
    square_thatch = 91
    circle_x = 92
    circle_x2 = 93
    circle_segment = 94
    empty1 = 95
    sword = 96
    horn_helmet = 97
    astro_helmet = 98
    target = 99
    empty2 = 100
    waves = 101
    rain = 102
    drink = 103
    paperclip = 104
    elephant = 105
    cowboyhat = 106
    ballcap = 107
    flag = 108
    bomb = 109
    bear = 110
    giraffe = 111
    bug1 = 112
    bug2 = 113
    bug3 = 114
    magnifying_glass = 115
    fishes = 116
    duck = 117
    sloth = 118
    bird = 119
    apple = 120
    carrot = 121
    lemon = 122
    pepper = 123
    onion = 124
    potion1 = 125
    potion2 = 126
    bow_and_arrow = 127
    campfire = 128
    donut = 129
    monitor = 130