from enum import IntEnum


class NamedColor(IntEnum):
    aliceblue = 0
    antiquewhite = 1
    aqua = 2
    aquamarine = 3
    azure = 4
    beige = 5
    bisque = 6
    black = 7
    blanchedalmond = 8
    blue = 9
    blueviolet = 10
    brown = 11
    burlywood = 12
    cadetblue = 13
    chartreuse = 14
    chocolate = 15
    coral = 16
    cornflowerblue = 17
    cornsilk = 18
    crimson = 19
    cyan = 20
    darkblue = 21
    darkcyan = 22
    darkgoldenrod = 23
    darkgray = 24
    darkgreen = 25
    darkgrey = 26
    darkkhaki = 27
    darkmagenta = 28
    darkolivegreen = 29
    darkorange = 30
    darkorchid = 31
    darkred = 32
    darksalmon = 33
    darkseagreen = 34
    darkslateblue = 35
    darkslategray = 36
    darkslategrey = 37
    darkturquoise = 38
    darkviolet = 39
    deeppink = 40
    deepskyblue = 41
    dimgray = 42
    dimgrey = 43
    dodgerblue = 44
    firebrick = 45
    floralwhite = 46
    forestgreen = 47
    fuchsia = 48
    gainsboro = 49
    ghostwhite = 50
    gold = 51
    goldenrod = 52
    gray = 53
    grey = 54
    green = 55
    greenyellow = 56
    honeydew = 57
    hotpink = 58
    indianred = 59
    indigo = 60
    ivory = 61
    khaki = 62
    lavender = 63
    lavenderblush = 64
    lawngreen = 65
    lemonchiffon = 66
    lightblue = 67
    lightcoral = 68
    lightcyan = 69
    lightgoldenrodyellow = 70
    lightgray = 71
    lightgreen = 72
    lightgrey = 73
    lightpink = 74
    lightsalmon = 75
    lightseagreen = 76
    lightskyblue = 77
    lightslategray = 78
    lightslategrey = 79
    lightsteelblue = 80
    lightyellow = 81
    lime = 82
    limegreen = 83
    linen = 84
    magenta = 85
    maroon = 86
    mediumaquamarine = 87
    mediumblue = 88
    mediumorchid = 89
    mediumpurple = 90
    mediumseagreen = 91
    mediumslateblue = 92
    mediumspringgreen = 93
    mediumturquoise = 94
    mediumvioletred = 95
    midnightblue = 96
    mintcream = 97
    mistyrose = 98
    moccasin = 99
    navajowhite = 100
    navy = 101
    oldlace = 102
    olive = 103
    olivedrab = 104
    orange = 105
    orangered = 106
    orchid = 107
    palegoldenrod = 108
    palegreen = 109
    paleturquoise = 110
    palevioletred = 111
    papayawhip = 112
    peachpuff = 113
    peru = 114
    pink = 115
    plum = 116
    powderblue = 117
    purple = 118
    red = 119
    rosybrown = 120
    royalblue = 121
    saddlebrown = 122
    salmon = 123
    sandybrown = 124
    seagreen = 125
    seashell = 126
    sienna = 127
    silver = 128
    skyblue = 129
    slateblue = 130
    slategray = 131
    slategrey = 132
    snow = 133
    springgreen = 134
    steelblue = 135
    tan = 136
    teal = 137
    thistle = 138
    tomato = 139
    turquoise = 140
    violet = 141
    wheat = 142
    white = 143
    whitesmoke = 144
    yellow = 145
    yellowgreen = 146
